--[[----------------------------------------------------------------------------

Lightroom Analytics by Chris Choy

------------------------------------------------------------------------------]]


local LrTasks = import 'LrTasks'
local catalog = import "LrApplication".activeCatalog()
      local LrHttp = import "LrHttp"
 local LrMD5 = import "LrMD5"
 local LrLogger = import 'LrLogger'
local LrStringUtils = import 'LrStringUtils'
local LrProgressScope = import 'LrProgressScope'

local myLogger = LrLogger( 'libraryLogger' ) -- the log file name
--myLogger:enable( "logfile" )

function outputToLog( message )
-- uncomment to enable log
--	myLogger:trace( message )
end


function print2( var, name )
  if not name then name = "anonymous" end
  if "table" ~= type( var ) then
    outputToLog( name .. " = " .. tostring( var ) )
  else
    -- for tables, recurse through children
    for k,v in pairs( var ) do
      local child
      if 1 == string.find( k, "%a[%w_]*" ) then
        -- key can be accessed using dot syntax
        child = name .. '.' .. k
      else
        -- key contains special characters
        child = name .. '["' .. k .. '"]'
      end
      print2( v, child )
    end
  end
end


  function newStack ()
      return {""}   -- starts with an empty string
    end
    
    function addString (stack, str)
      table.insert(stack, str)    -- push 's' into the the stack
      for i=(#stack)-1, 1, -1 do
        if string.len(stack[i]) > string.len(stack[i+1]) then
          break
        end
        stack[i] = stack[i] .. table.remove(stack)
      end
    end

  function stackToString(stack)
      local output = ''
      for i=1, #stack, 1 do
        output = table.remove(stack)..output
      end
    return output
  end


  function newLRData()
    return {}
  end

  function addLRData(data,key,value)
    if data[key]==nil then
      data[key]={}
    end
    table.insert(data[key],value)
  end

  function dataToJSON(data,mys)
--    local mys = newStack()
    addString(mys,"{")
    for key,value in pairs(data) do 
      addString(mys, "\""..key.."\":[")
      for i,value in ipairs(data[key]) do
         if i==1 then 
            addString(mys,"\""..value.."\"")
         else
            addString(mys,",\""..value.."\"")
         end

      end
      addString(mys,"],")
    end
    addString(mys,"\"version\":3}")
    --return stackToString(mys)
  end

  function dataToCSV(data,mys)
--    local mys = newStack()
    for key,value in pairs(data) do 
      addString(mys, "\""..key.."\",")
      for i,value in ipairs(data[key]) do
         if i==1 then 
            addString(mys,"\""..value.."\"")
         else
            addString(mys,",\""..value.."\"")
         end

      end
      addString(mys,"\n")
    end
    
    --return stackToString(mys)
  end
function mainfunction()
	local output = ''
	local s = newStack()
        local lrdata = newLRData()
	outputToLog('begin')
	LrTasks.startAsyncTask(function()
		catalog:withWriteAccessDo( "Preparing Dataset", function()
			local cat_photos = catalog:getTargetPhotos()
			local progress = LrProgressScope({title="Crunching Metadata"})
			progress:setCancelable(true)
			-- loop through each of the photos
			for i, photo in ipairs(cat_photos) do
				progress:setPortionComplete(i,#cat_photos)
				if progress:isCanceled() then return end
				outputToLog(i)
			        local rawmetatags = {"shutterSpeed","focalLength35mm","focalLength","aperture","rating","exposureBias","flash","isoSpeedRating"}
--print2(photo:getRawMetadata())
--print2(photo:getFormattedMetadata())
				local tmp
				for j,tag in ipairs(rawmetatags) do
					tmp = photo:getRawMetadata(tag)
					if tmp==nil then
						addLRData(lrdata,tag,"NA")
						--addString(s,tag.."[]=NA&")
					else
						if tmp==true then
							addLRData(lrdata,tag,"true")
							--addString(s,tag.."[]=true&")
						elseif tmp==false then
							addLRData(lrdata,tag,"false")
							--addString(s,tag.."[]=false&")
						else
							addLRData(lrdata,tag,tmp)
							--addString(s,tag.."[]="..tmp.."&")
						end
					end
				end
				local rawmetatags = {"fileName","lens","cameraMake","cameraModel","subjectDistance","exposureProgram","meteringMode"}
				--local rawmetatags = {"lens","cameraMake","cameraModel","subjectDistance","exposureProgram","meteringMode"}
				for j,tag in ipairs(rawmetatags) do
					tmp = photo:getFormattedMetadata(tag)
					if tmp==nil then
						addLRData(lrdata,tag,"NA")
					else
						if tmp==true then
							addLRData(lrdata,tag,"true")
						elseif tmp==false then
							addLRData(lrdata,tag,"false")
						else
							if key=="fileName" then
								tmp = LrStringUtils.encodeBase64(tmp)
								addLRData(lrdata,tag,tmp)
							else
								addLRData(lrdata,tag,tmp)
							end

						end
					end
				end
				local devsettings = photo:getDevelopSettings()



				for tag,tmp in pairs(devsettings) do

					if tmp==nil then
						addLRData(lrdata,tag,"NA")
					else
						if tmp==true then
							addLRData(lrdata,tag,"true")
						elseif tmp==false then
							addLRData(lrdata,tag,"false")
						else
							if type(tmp)~="table" then
								addLRData(lrdata,tag,tmp) 
							end						
						end
					end
				end
			end

		outputToLog('end output')
                --addString(s,'totalimage='..#cat_photos..'&')
                --addString(s,'jsonstring=')
                dataToJSON(lrdata,s)
		output = stackToString(s)
		outputToLog(output)
		
		
		local LrPathUtils = import "LrPathUtils"
		local file = io.open(LrPathUtils.child(LrPathUtils.getStandardFilePath("desktop"),"lranalytics.json"), "w")
		file:write(output)
		file:close()

		s = newStack()
                dataToCSV(lrdata,s)
		output = stackToString(s)
		file = io.open(LrPathUtils.child(LrPathUtils.getStandardFilePath("desktop"),"lranalytics.csv"), "w")
		file:write(output)
		file:close()

		progress:done()


		end)
	end)
end

mainfunction()
